@@ -324,8 +324,9 @@ def enc_apl(bng, x, met):
             mi = 1
         cnd_st = [None]*mi
         xe = nans((N, mi))
+        hh = 0
         for h in range(1, len(bns) + 1):
-            if met == 'dv' and not bns[h]['n'] > 0: np.delete(xe, h, axis=1); continue
+            if met == 'dv' and not bns[h]['n'] > 0: xe = np.delete(xe, h-1-hh, axis=1); hh += 1; continue
             dv = b2dv(bns[h], xi, xtp)==1
             if met == 'my':      xe[dv] = bns[h]['my'];  cname_e = cname + '_encMY'
             elif met == 'woe':   xe[dv] = bns[h]['woe'];  cname_e = cname + '_encWOE'
