@@ -10,6 +10,22 @@ pd.set_option('display.max_columns', 500)
 pd.set_option('display.width', 1000)
 
 
+###################################################################################
+def add_model(model, model_1, FB):
+    if FB == 'FS':
+        AIC = model['ovr']['AIC']
+        BIC = model['ovr']['BIC']
+    else:
+        AIC = model['st']['ovr']['AIC']
+        BIC = model['st']['ovr']['BIC']
+    AIC_1 = model_1['st']['ovr']['AIC']
+    BIC_1 = model_1['st']['ovr']['BIC']
+    if AIC > AIC_1: better = 0
+    else:           better = 1
+    if BIC > BIC_1: better = 0
+    else:           better = 1
+
+    return better
 ###################################################################################
 def bElimin(model, step, x, y, w, met, SLS, pm0, st0, cnames, metpminit, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, dsp,
             iterate, sc, dsp_op, smeth, su, p_decr, p_incr, smin):
@@ -33,17 +49,23 @@ def bElimin(model, step, x, y, w, met, SLS, pm0, st0, cnames, metpminit, met_opt
                          gcnv, sc, dsp_op, smeth, su, p_decr, p_incr)
             ym = lgr_apl(x[:, ivi], model1['pm'])
             st_ext = stmdl('FS1', x, y, ym, w, st0, model1, n2 - 1, ivi, ivo, smin)
-            model = model + [htmdl('BE1', [], 'BE', n2 - 1, ivi, ivo, cnames, ivi0[ind], model1['pm'], st_ext)]
-            visualz(model, iterate, dsp)
-            for i in np.arange(0, len(model) - 1):
-                if len(model[i]['ivi']) == len(model[-1]['ivi']) and all(
-                    model[i]['ivi'] == model[-1]['ivi']): iterate = 0
-            step = 'BE'
+
+            better_mdl = add_model(model1, model[-1], 'BE')
+            if better_mdl:
+                model = model + [htmdl('BE1', [], 'BE', n2 - 1, ivi, ivo, cnames, ivi0[ind], model1['pm'], st_ext)]
+                visualz(model, iterate, dsp)
+                for i in np.arange(0, len(model) - 1):
+                    if len(model[i]['ivi']) == len(model[-1]['ivi']) and all(
+                        model[i]['ivi'] == model[-1]['ivi']): iterate = 0
+                step = 'BE'
+            else:
+                if met == 'BR' or step == 'FS_STOP': iterate = 0
+                step = 'BE_STOP'
         else:
-            if met == 'BR' or step == 'FE_STOP': iterate = 0
+            if met == 'BR' or step == 'FS_STOP': iterate = 0
             step = 'BE_STOP'
     else:
-        if met == 'BR' or step == 'FE_STOP': iterate = 0
+        if met == 'BR' or step == 'FS_STOP': iterate = 0
         step = 'BE_STOP'
 
     return model, step, st0, iterate
@@ -108,18 +130,24 @@ def fSelect(model, step, x, y, w, met, SLE, pm0, st0, cnames, metpminit, met_opt
                          gcnv, sc, dsp_op, smeth, su, p_decr, p_incr)
             ym = lgr_apl(x[:, ivi], model1['pm'])
             st_ext = stmdl('FS1', x, y, ym, w, st0, model1, n2 + 1, ivi, ivo, smin)
-            model = model + [htmdl('FS1', [], 'FS', n2 + 1, ivi, ivo, cnames, ivi[-1], model1['pm'], st_ext)]
-            visualz(model, iterate, dsp)
-            for i in np.arange(0, len(model) - 1):
-                if len(model[i]['ivi']) == len(model[-1]['ivi']) and all(
-                    model[i]['ivi'] == model[-1]['ivi']): iterate = 0
-            step = 'FS'
+
+            better_mdl = add_model(st_ext, model[-1], 'FS')
+            if better_mdl:
+                model = model + [htmdl('FS1', [], 'FS', n2 + 1, ivi, ivo, cnames, ivi[-1], model1['pm'], st_ext)]
+                visualz(model, iterate, dsp)
+                for i in np.arange(0, len(model) - 1):
+                    if len(model[i]['ivi']) == len(model[-1]['ivi']) and all(
+                        model[i]['ivi'] == model[-1]['ivi']): iterate = 0
+                step = 'FS'
+            else:
+                if met == 'FR' or step == 'BE_STOP': iterate = 0
+                step = 'FS_STOP'
         else:
             if met == 'FR' or step == 'BE_STOP': iterate = 0
-            step = 'FE_STOP'
+            step = 'FS_STOP'
     else:
         if met == 'FR' or step == 'BE_STOP': iterate = 0
-        step = 'FE_STOP'
+        step = 'FS_STOP'
 
     return model, step, st0, iterate
 
@@ -189,7 +217,7 @@ def mdl(x, y, w, ivi, n2, st0, pm0, cnames, metpminit, meth, maxiter, fcnv, afcn
     func = 1
     grad = 1
     hes = 1
-    pm, F, g, H, Hinv, __ = newton(pm, f_lgr, g_lgr, h_lgr, args, meth, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, smeth, su, p_decr, p_incr)
+    pm, F, g, H, Hinv, __ = newton(pm, f_lgr, g_lgr, h_lgr, args, meth, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, smeth, su, p_incr, p_decr)
     # # Denormalization
     # pm = denormPar(dset, pmn);
 
@@ -257,7 +285,7 @@ def scrChiSq(x, y, ym0, ym, w, st0, model, mode, ivi, ivo, smin):
         if nci == 0:
             S = np.nan
             return S
-        gin = -g
+        gin = g
         if nco == 0:
             S = c_(np.array([]))
             return S
@@ -268,14 +296,20 @@ def scrChiSq(x, y, ym0, ym, w, st0, model, mode, ivi, ivo, smin):
             if not any(xout - xout[0]):
                 S[i, 0] = 0
                 continue
-            gout = xout.T @ ((y - ym)*w)
-            Hi = -(x[:, fi[i, :]]*ym*(1 - ym)*w).T @ x[:, fi[i, :]]
+            gout = -xout.T @ ((y - ym)*w)
+
+            Hi = (x[:, fi[i, :]]*ym*(1 - ym)*w).T @ x[:, fi[i, :]]
             g = np.vstack((gin, gout))
+
+            # args = {'data': np.hstack((x[:, fi[i, :]], y, w, ym))}
+            # g = g_lgr(args)
+            # Hi = h_lgr(args)
             rcHi = 1/np.linalg.cond(Hi)
-            if rcHi < 1e-6:
-                S[i, 0] = (g.T @ (-nsinv(Hi, smin)) @ g)[0, 0]
+            if rcHi < np.inf: # 1e-6:
+                S[i, 0] = (g.T@(nsinv(Hi, smin))@g)[0, 0]
             else:
-                S[i, 0] = (g.T @ (-inv2(Hi, Hinv, smin)) @ g)[0, 0]
+                pass
+                # S[i, 0] = (g.T@(inv2(Hi, Hinv, smin))@g)[0, 0]
         S[S < 0] = 0
     return S
 
@@ -292,10 +326,8 @@ def swlogr(x, y, w, cnames=None, SLE=0.05, SLS=0.05, pm0=1, dsp='all', nbm_crit=
         cnames = ['intercept'] + cnames
         ivi = np.hstack((1, ivi + 1))
     iterate = 1
-    st0 = stats(x, y, w, smin, pm0, cnames, metpminit, ivi, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, dsp,
-                smeth, su, p_decr, p_incr)
-    model, step, st0 = firstmdl(x, y, w, cnames, st0, smin, mtp, ivi, pm0, metpminit, met_opt, maxiter, fcnv, afcnv,
-                                xcnv, gcnv, sc, dsp_op, smeth, su, p_decr, p_incr)
+    st0 = stats(x, y, w, smin, pm0, cnames, metpminit, ivi, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, dsp, smeth, su, p_decr, p_incr)
+    model, step, st0 = firstmdl(x, y, w, cnames, st0, smin, mtp, ivi, pm0, metpminit, met_opt, maxiter, fcnv, afcnv, xcnv, gcnv, sc, dsp_op, smeth, su, p_decr, p_incr)
     while iterate:
         model, step, st0, iterate = bElimin(model, step, x, y, w, met, SLS, pm0, st0, cnames, metpminit, met_opt,
                                             maxiter, fcnv, afcnv, xcnv, gcnv, dsp, iterate, sc, dsp_op, smeth, su,
@@ -607,12 +639,15 @@ def visualz(model, iterate, dsp):
         No = np.arange(1, len(ScoreChiSq) + 1)
         DF = np.ones((ScoreChiSq.shape[0],))
         Variable = model['cname_o'].flatten()
-        print(pd.DataFrame({'No': No,
-                            'Variable': Variable,
-                            'DF': DF,
-                            'ScoreChiSq': ScoreChiSq,
-                            'p_valScoreChiSq': p_valScoreChiSq
-                            }))
+        if len(ScoreChiSq) > 0:
+            print(pd.DataFrame({'No': No,
+                                'Variable': Variable,
+                                'DF': DF,
+                                'ScoreChiSq': ScoreChiSq,
+                                'p_valScoreChiSq': p_valScoreChiSq
+                                }))
+        else:
+            print('No factors are eligible for entry...')
         print('------------------------------------------------------')
         print(' ')
 
