@@ -38,14 +38,16 @@ def f_cs(X, args):
     F = np.full((N,), np.nan)
     i = 0
     for x in X:
-        x1, ss = sort(c_(x[:nc1, ]))
-        x2, ss = sort(c_(x[nc1:, ]))
-        minx = np.min([minx, np.min(x)])  # upper bound for cut-offs of the first scorecard
-        maxx = np.max([maxx, np.max(x)])  # upper bound for cut-offs of the second scorecard
-        x1 = (x1 - minx) / (maxx - minx) * (ub1 - lb1) + lb1
-        x2 = (x2 - minx) / (maxx - minx) * (ub2 - lb2) + lb2
-        x1 = np.round(x1.flatten())
-        x2 = np.round(x2.flatten())
+        x1 = x[:nc1, ]
+        x2 = x[nc1:, ]
+        # x1, ss = sort(c_(x[:nc1, ]))
+        # x2, ss = sort(c_(x[nc1:, ]))
+        # minx = np.min([minx, np.min(x)])  # upper bound for cut-offs of the first scorecard
+        # maxx = np.max([maxx, np.max(x)])  # upper bound for cut-offs of the second scorecard
+        # x1 = (x1 - minx) / (maxx - minx) * (ub1 - lb1) + lb1
+        # x2 = (x2 - minx) / (maxx - minx) * (ub2 - lb2) + lb2
+        # x1 = np.round(x1.flatten())
+        # x2 = np.round(x2.flatten())
         if len(set(x1)) != len(x1) or len(set(x2)) != len(x2): return case*np.inf  # if there are 2 or more the same GB cut-offs: F = inf
         try:
             GB_zones = cut(data['Score_GB'].values, x1).flatten()
